/*
    Open Source BDM - processing of commands received over USB
	  Prominent Notice-This software was modified from TBDML software - 12/05

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "MC68HC908JB16.h"
#include "bdm.h"
#include "cmd_processing.h"
#include "commands.h"
#include "version.h"
#include "usb.h"

#include "led.h"
extern volatile char state_global;
unsigned char bcommand_buffer[0x44] @ 0x101;
extern char far __SEG_START_SSTACK[];
         

// extern addr1;
/* processes all commands received over USB */
/* the command is expected to be in command_buffer+1  */
/* returns number of bytes left in the buffer to be sent back as response */
unsigned char command_exec(void) {

  unsigned char bigEndianWordBuf[2];
  unsigned char bigEndianLongWordBuf[4];
  unsigned char command_buffer_0_org;
  unsigned char cfv1_xcsr_byte;

  led_state = LED_BLINK;                          /* blink the LED to indicate a command */
  if (bdm_status.target_type==HC12) {
    if (command_buffer[1]==CMD_GET_LAST_STATUS) {
      /* need to process this special command before status of the last command is lost */
      return(1);      
    } 
  }
  //===========================================
  else if ((bdm_status.target_type==HCS08)||(bdm_status.target_type==RS08)||(bdm_status.target_type==CFV1)) {
    if (command_buffer[1]==CMD_GET_LAST_STATUS) {
      /* need to process this special command before status of the last command is lost */
      return(1);      
    } else {
    
      command_buffer_0_org = command_buffer[0];
      command_buffer[0] = command_buffer[1];      /* assume the command will execute OK */
      switch (command_buffer[1]) {
        /* get HW & SW version */
        case CMD_GET_VER:
          *((unsigned int *)(command_buffer+1)) = VERSION;
          return(3);                              /* return cmd + 2 bytes of version */
        /* set target type */                            
        case CMD_SET_TARGET:
          bdm_status.target_type = command_buffer[2];
          return(1);                              /* confirm reception by transmitting cmd number back */
        /* set derivative type */                            
        case CMD_SET_DERIVATIVE:
         bdm_status.derivative_type = command_buffer[2];
          return(1);
  			/* try to connect to the target */
        case CMD_CONNECT:
          if (bdm08_connect()) {
            command_buffer[0] = CMD_FAILED;       /* cannot connect */
          }
         return(1);
 
    
        /* reset */
        case CMD_RESET:
        		if ((bdm_status.target_type==RS08)){
        		        bdm_rs08softreset(command_buffer[2]);
        		        return(1); 
        		}
         		else {
          		bdm_softreset(command_buffer[2]);
     
   /*       	if (bdm08_stat()==0) {    
            			 if (bdm_reset(command_buffer[2])) {
                    command_buffer[0] = CMD_FAILED;       // reset line stuck 
                  }				
          		} 
          		else {
          					 bdm_softreset(command_buffer[2]);        				
       				}  
    */   				
              return(1); 
        		}

        /* return status of BDM communication */
        case CMD_GET_STATUS:
          command_buffer[1]=0;  /* intel LSB */
          if (bdm_status.ackn==ACKN) command_buffer[1]|=0x01; /* target supports ACKN and the feature is enabled */
          if (bdm_status.reset==RESET_DETECTED) {
            command_buffer[1]|=0x02;                          /* the target was resently reset externaly */
            if (RESET_IN==1) bdm_status.reset=NO_RESET_ACTIVITY; /* clear the flag if reset pin has gone high already */
          }
          if (bdm_status.speed==SPEED_USER_SUPPLIED) command_buffer[1]|=0x18; else  /* do not blame me if it does not work... */
            if (bdm_status.speed==SYNC_SUPPORTED) command_buffer[1]|=0x08; else     /* SYNC feature used to detect speed */
              if (bdm_status.speed==SPEED_GUESSED) command_buffer[1]|=0x10;         /* speed guessed by trial & error */
          command_buffer[2]=0;  /* intel MSB */
          return(2);
        case CMD_READ_STATUS: 
          BDM08_CMD_READSTATUS(command_buffer+1);
          return(2);     
        case CMD_WRITE_CONTROL: 
          BDM08_CMD_WRITECONTROL(command_buffer[2]);
          return(2);    
        /* stop the target */
        
        case CMD_HALT:
          if (bdm_status.target_type==CFV1) {
          		bdm_sync_meas();
		          bdm_rx_tx_select();
              BDMCFV1_CMD_READ_XCSR_BYTE(&cfv1_xcsr_byte);
              if ((cfv1_xcsr_byte&0x01)==0) 
                  BDMCFV1_CMD_WRITE_XCSR_BYTE(0x01|cfv1_xcsr_byte); /* if BDM not enabled yet, enable it so it can be made active */
        
              BDMCFV1_CMD_BACKGROUND();
          } 
          else{
              BDM_CMD_BACKGROUND();
          }
          return(1);
          
        case CMD_MASS_ERASE:
         // bdm_mass_erase();
          return(0) ;
        /* set comm speed to user supplied value */
        case CMD_SET_SPEED1: 
          {
            unsigned char bdm_sts;
            bdm_status.sync_length=(unsigned int)256*command_buffer[3]+command_buffer[2]; /* save the speed */
            if (bdm_rx_tx_select()) {
              command_buffer[0] = CMD_FAILED;       /* no Rx or Tx routine for this speed */
              bdm_status.speed = NO_INFO;           /* connection cannot be established at this speed */
              bdm_status.ackn = WAIT;               /* clear indication of ACKN feature */
              return(1);
            }
            bdm_status.speed = SPEED_USER_SUPPLIED; /* user told us */
             bdm_ackn_init();                        /* try the ACKN feature */     
          	 BDM08_CMD_READSTATUS(&bdm_sts);
 
          	if ((bdm_sts&0x80)==0) BDM12_CMD_BDWRITEB(BDM12_STS_ADDR,0x80|bdm_sts);	/* if BDM not enabled yet, enable it so it can be made active */
            return(1);          
          }
        /* Read current speed */
        case CMD_READ_SPEED1:
          command_buffer[2] = (bdm_status.sync_length)>>8;   /* store speed in little endian */
          command_buffer[1] = (bdm_status.sync_length)&0xff;
          return(3);
        /* Start code execution */
        case CMD_GO1:
          if (bdm_status.target_type==CFV1) {
              bdm_sync_meas();
		          bdm_rx_tx_select();
              BDMCFV1_CMD_READ_XCSR_BYTE(&cfv1_xcsr_byte);
              if ((cfv1_xcsr_byte&0x01)==0) 
                  BDMCFV1_CMD_WRITE_XCSR_BYTE(0x01|cfv1_xcsr_byte); /* if BDM not enabled yet, enable it so it can be made active */
           } 

            BDM_CMD_GO();
            return(1);
        /////////////////////////////  
       case CMD_FLASH_DLSTART:
             BDM_CMD_BACKGROUND();  
            
         if (bdm_status.derivative_type == LA8){
             BDM08_CMD_WRITEB(0x22D,0x40);    //ICSC2
             BDM08_CMD_WRITEB(0x22E,0x80);    //ICSTRM
         } 
         else{
             BDM08_CMD_WRITEB(0x15,0x40);    //ICSC2
             BDM08_CMD_WRITEB(0x16,0x80);    //ICSTRM
          
         }
         
          bdm08_connect();   
            //Turn VPP On
 	        PTD_PTD4 = 0;  
          bdm_mass_erase();
          
          state_global = 2;  
          command_buffer[0]=CMD_FAILED;
          return(0);   
          
          /////////////////////////////////////////
        /* Step over 1 instruction */
 
        case CMD_STEP1: 
        {
            if (bdm_status.target_type==CFV1) {
                bdm_sync_meas();
		            bdm_rx_tx_select();
                BDMCFV1_CMD_READ_XCSR_BYTE(&cfv1_xcsr_byte);
                if ((cfv1_xcsr_byte&0x01)==0) 
                   BDMCFV1_CMD_WRITE_XCSR_BYTE(0x01|cfv1_xcsr_byte); /* if BDM not enabled yet, enable it so it can be made active */

                // Read CSR register (crn =0)
                BDMCFV1_CMD_READ_DREG(0, (unsigned char*)bigEndianLongWordBuf);
            
                // Set SSM (Signle Step Mode) bit of CSR
                bigEndianLongWordBuf[3] = bigEndianLongWordBuf[3] | 0x10;
                BDMCFV1_CMD_WRITE_DREG(0, *(unsigned long*)((void*)bigEndianLongWordBuf));
                
                // The processor will halt after execution of the next instruction when SSM is set.
                BDM_CMD_GO();
            
                // Read CSR register (crn =0)
                BDMCFV1_CMD_READ_DREG(0, (unsigned char*)bigEndianLongWordBuf);

                 // Clear SSM (Signle Step Mode) bit of CSR
                bigEndianLongWordBuf[3] = bigEndianLongWordBuf[3] & (~(0x10));
                BDMCFV1_CMD_WRITE_DREG(0, *(unsigned long*)((void *)bigEndianLongWordBuf));
            } 
            else{
              
              BDM_CMD_TRACE1();
            }
           
            return(1);
         }
          
          
        /* read data/address register for CFV1 */  
        case CMD_READ_REG_32: 
        {
            bdm_sync_meas();
		        bdm_rx_tx_select();
            BDMCFV1_CMD_READ_XCSR_BYTE(&cfv1_xcsr_byte);
            if ((cfv1_xcsr_byte&0x01)==0) 
                BDMCFV1_CMD_WRITE_XCSR_BYTE(0x01|cfv1_xcsr_byte); /* if BDM not enabled yet, enable it so it can be made active */
      
            BDMCFV1_CMD_READ_Rn((unsigned char)command_buffer[2],(unsigned char*)bigEndianLongWordBuf);
            //* send the value with little endian order */
            command_buffer[1] = bigEndianLongWordBuf[3];
            command_buffer[2] = bigEndianLongWordBuf[2];
            command_buffer[3] = bigEndianLongWordBuf[1];
            command_buffer[4] = bigEndianLongWordBuf[0];
            return(5);
        }
        
        /* read control register for CFV1 */  
        case CMD_READ_CREG_32:
        
           	bdm_sync_meas();
		        bdm_rx_tx_select();
            BDMCFV1_CMD_READ_XCSR_BYTE(&cfv1_xcsr_byte);
            if ((cfv1_xcsr_byte&0x01)==0) 
                BDMCFV1_CMD_WRITE_XCSR_BYTE(0x01|cfv1_xcsr_byte); /* if BDM not enabled yet, enable it so it can be made active */
          
            BDMCFV1_CMD_READ_CREG((unsigned char)command_buffer[2],(unsigned char*)bigEndianLongWordBuf);
            //* send the value with little endian order */
            command_buffer[1] = bigEndianLongWordBuf[3];
            command_buffer[2] = bigEndianLongWordBuf[2];
            command_buffer[3] = bigEndianLongWordBuf[1];
            command_buffer[4] = bigEndianLongWordBuf[0];
            return(5);

        /* read debug register for CFV1 */  
        case CMD_READ_DREG_32:
        
            bdm_sync_meas();
		        bdm_rx_tx_select();
            BDMCFV1_CMD_READ_XCSR_BYTE(&cfv1_xcsr_byte);
            if ((cfv1_xcsr_byte&0x01)==0) 
                BDMCFV1_CMD_WRITE_XCSR_BYTE(0x01|cfv1_xcsr_byte); /* if BDM not enabled yet, enable it so it can be made active */

            BDMCFV1_CMD_READ_DREG((unsigned char)command_buffer[2],(unsigned char*)bigEndianLongWordBuf);
            //* send the value with little endian order */
            command_buffer[1] = bigEndianLongWordBuf[3];
            command_buffer[2] = bigEndianLongWordBuf[2];
            command_buffer[3] = bigEndianLongWordBuf[1];
            command_buffer[4] = bigEndianLongWordBuf[0];
            return(5);

        /* read XCSR register for CFV1.  Other targets uses CMD_READ_STATUS. */  
        case CMD_READ_XCSR_8:
        
            bdm_sync_meas();
		        bdm_rx_tx_select();
		        
            BDMCFV1_CMD_READ_XCSR_BYTE(command_buffer+1);
            return(2);
     
    
        /* read CSR2 register for CFV1. */  
        case CMD_READ_CSR2_8:
        
            bdm_sync_meas();
		        bdm_rx_tx_select();
		        
            BDMCFV1_CMD_READ_CSR2_BYTE(command_buffer+1);
            return(2);

   
        /* read CSR3 register for CFV1. */  
        case CMD_READ_CSR3_8:
        
            bdm_sync_meas();
		        bdm_rx_tx_select();
		        
            BDMCFV1_CMD_READ_CSR3_BYTE(command_buffer+1);
            return(2);
            
        /* write XCSR register for CFV1 */
        case CMD_WRITE_XCSR_8:
            bdm_sync_meas();
		        bdm_rx_tx_select();
		        
            BDMCFV1_CMD_WRITE_XCSR_BYTE(command_buffer[2]);
            return(1);

      /* write CSR2 register for CFV1 */
        case CMD_WRITE_CSR2_8:
            bdm_sync_meas();
		        bdm_rx_tx_select();
		        
            BDMCFV1_CMD_WRITE_CSR2_BYTE(command_buffer[2]);
            return(1);

      /* write XCSR register for CFV1 */
        case CMD_WRITE_CSR3_8:
            bdm_sync_meas();
		        bdm_rx_tx_select();
		        
            BDMCFV1_CMD_WRITE_CSR3_BYTE(command_buffer[2]);
            return(1);


        /* write data/address register for CFV1 */  
        case CMD_WRITE_REG_32:
        
            bdm_sync_meas();
		        bdm_rx_tx_select();
            BDMCFV1_CMD_READ_XCSR_BYTE(&cfv1_xcsr_byte);
            if ((cfv1_xcsr_byte&0x01)==0) 
                BDMCFV1_CMD_WRITE_XCSR_BYTE(0x01|cfv1_xcsr_byte); /* if BDM not enabled yet, enable it so it can be made active */

            bigEndianLongWordBuf[0] = command_buffer[6];
            bigEndianLongWordBuf[1] = command_buffer[5];
            bigEndianLongWordBuf[2] = command_buffer[4];
            bigEndianLongWordBuf[3] = command_buffer[3];
      
            BDMCFV1_CMD_WRITE_Rn((unsigned char)command_buffer[2],*(unsigned long*)(bigEndianLongWordBuf));
            return(1);
             
     
        /* write control register for CFV1 */  
        case CMD_WRITE_CREG_32:
        
            bdm_sync_meas();
		        bdm_rx_tx_select();
            BDMCFV1_CMD_READ_XCSR_BYTE(&cfv1_xcsr_byte);
            if ((cfv1_xcsr_byte&0x01)==0) 
                BDMCFV1_CMD_WRITE_XCSR_BYTE(0x01|cfv1_xcsr_byte); /* if BDM not enabled yet, enable it so it can be made active */

            bigEndianLongWordBuf[0] = command_buffer[6];
            bigEndianLongWordBuf[1] = command_buffer[5];
            bigEndianLongWordBuf[2] = command_buffer[4];
            bigEndianLongWordBuf[3] = command_buffer[3];
      
            BDMCFV1_CMD_WRITE_CREG((unsigned char)command_buffer[2],*(unsigned long*)(bigEndianLongWordBuf));
            return(1);
  
        /* write debug register for CFV1 */  
        case CMD_WRITE_DREG_32:
        
            bdm_sync_meas();
		        bdm_rx_tx_select();
            BDMCFV1_CMD_READ_XCSR_BYTE(&cfv1_xcsr_byte);
            if ((cfv1_xcsr_byte&0x01)==0) 
                BDMCFV1_CMD_WRITE_XCSR_BYTE(0x01|cfv1_xcsr_byte); /* if BDM not enabled yet, enable it so it can be made active */

            bigEndianLongWordBuf[0] = command_buffer[6];
            bigEndianLongWordBuf[1] = command_buffer[5];
            bigEndianLongWordBuf[2] = command_buffer[4];
            bigEndianLongWordBuf[3] = command_buffer[3];
  
            BDMCFV1_CMD_WRITE_DREG((unsigned char)command_buffer[2],*(unsigned long*)(bigEndianLongWordBuf));
            return(1);

        /* read memory for CFV1 */
        case CMD_READ_BLOCK_WITH_ACCESSTYPE: 
        {
                
            unsigned long addr;                                 
            unsigned char numBytes = command_buffer_0_org-1;
            unsigned char numWords = numBytes/2;
            unsigned char numLongWords = numBytes/4;
          
            unsigned char numBytes_org = command_buffer_0_org-1;
            unsigned char accessType = command_buffer[5];
            unsigned char *data_ptr = command_buffer+1;
 
            bigEndianLongWordBuf[0] = 0;
            bigEndianLongWordBuf[1] = command_buffer[4];
            bigEndianLongWordBuf[2] = command_buffer[3];
            bigEndianLongWordBuf[3] = command_buffer[2];
            addr = *(unsigned long*)(bigEndianLongWordBuf);
        
            if (numBytes>COMMAND_BUFFER_SIZE) {
              command_buffer[0] = CMD_FAILED; /* requested block is too long to fit into the buffer */
              return(1);  
            }
            
            bdm_sync_meas();
		        bdm_rx_tx_select();
            BDMCFV1_CMD_READ_XCSR_BYTE(&cfv1_xcsr_byte);
            if ((cfv1_xcsr_byte&0x01)==0) 
                BDMCFV1_CMD_WRITE_XCSR_BYTE(0x01|cfv1_xcsr_byte); /* if BDM not enabled yet, enable it so it can be made active */

            switch(accessType){
           
              case 1:  
                 /* NOTE: This command returns block of BYTES, endianism does not matter here. */
                 
                while(numBytes&0xfe) {
                   /* while there are at least 2 bytes to fetch */
                  BDMCFV1_CMD_READ_MEM_BYTE((unsigned long)addr,(unsigned char*)data_ptr);    
                  addr+=1;                           /* increment address */
                  data_ptr+=1;                       /* increment pointer */
                  numBytes-=1;                       /* decrement count */
                }
                if (numBytes) {
                  /* there is one extra byte to read at the end of the block */  
                   BDMCFV1_CMD_READ_MEM_BYTE((unsigned long)addr,(unsigned char*)data_ptr);   
           
                }
                return(numBytes_org+1);
 
             case 2: 
                while(numWords&0xfe) {
                  BDMCFV1_CMD_READ_MEM_WORD((unsigned long)addr,(unsigned char*)(data_ptr));  
                  addr+=2;                           /* increment address */
                  data_ptr+=2;                       /* increment pointer */
                  numWords-=1;                       /* decrement count */
                }
                if (numWords) {
                  /* there is one extra word to read at the end of the block */  
                  BDMCFV1_CMD_READ_MEM_WORD((unsigned long)addr,(unsigned char*)(data_ptr));  
                }
                return(numBytes_org+1);

            case 4:  
                do{
                  
                  BDMCFV1_CMD_READ_MEM_LONG((unsigned long)addr,(unsigned char*)(data_ptr));
 
                  addr+=4;                           /* increment address */
                  data_ptr+=4;                       /* increment pointer */
                  numLongWords-=1;                   /* decrement count */
                }   while(numLongWords);

                return(numBytes_org+1);
            }
        }
        
        
        /* write memory for CFV1 */
        case CMD_WRITE_BLOCK_WITH_ACCESSTYPE: 
        {
            int i1;
            unsigned long addr;
            unsigned char numBytes = command_buffer_0_org;
    
            unsigned char accessType = command_buffer[5];
            unsigned char *data_ptr = (unsigned char*)(&command_buffer[6]);
           
            unsigned char numLongWords = numBytes/4;
            unsigned char numWords = numBytes/2;
         
                                                  
            bigEndianLongWordBuf[0] = 0;
            bigEndianLongWordBuf[1] = command_buffer[4];
            bigEndianLongWordBuf[2] = command_buffer[3];
            bigEndianLongWordBuf[3] = command_buffer[2];
            addr = *(unsigned long*)(bigEndianLongWordBuf);
    
            bdm_sync_meas();
		        bdm_rx_tx_select();
            BDMCFV1_CMD_READ_XCSR_BYTE(&cfv1_xcsr_byte);
            if ((cfv1_xcsr_byte&0x01)==0) 
                BDMCFV1_CMD_WRITE_XCSR_BYTE(0x01|cfv1_xcsr_byte); /* if BDM not enabled yet, enable it so it can be made active */
        
            switch(accessType){
           
              case 1:  
                 /* NOTE: This command returns block of BYTES, endianism does not matter here. */
                 
                while(numBytes&0xfe) {
                  /* while there are at least 2 bytes to fetch */
                   BDMCFV1_CMD_WRITE_MEM_BYTE((unsigned long)addr,*(unsigned char *)(data_ptr));  
                   addr+=1;                              /* increment address */
                   data_ptr+=1;                          /* increment pointer */
                   numBytes-=1;                          /* decrement count */
                }
                if (numBytes) {
                  /* there is one extra byte to write at the end of the block */  
                   BDMCFV1_CMD_WRITE_MEM_BYTE((unsigned long)addr,*(unsigned char *)(data_ptr)); 
                } 
                return(1);
                
              case 2:
                     
                while(numWords&0xfe) {
                  /* while there are at least 2 words to fetch */
                   BDMCFV1_CMD_WRITE_MEM_WORD((unsigned long)addr,*(unsigned int *)(data_ptr));  

                   addr+=2;                              /* increment address */
                   data_ptr+=2;                          /* increment pointer */
                   numWords-=1;                          /* decrement count */
                }
                if (numWords) {
                   /* there is one extra word to write at the end of the block */
                    BDMCFV1_CMD_WRITE_MEM_WORD((unsigned long)addr,*(unsigned int *)(data_ptr));  
                } 
                return(1);
                
              case 4: 
              {
                 unsigned long value;
                 do {
                   BDMCFV1_CMD_WRITE_MEM_LONG((unsigned long)addr,*(unsigned long*)(data_ptr));  
                  
                   addr+=4;                              /* increment address */
                   data_ptr+=4;                          /* increment pointer */
                   numLongWords-=1;                      /* decrement count */
                } while(numLongWords);
           
                return(1);
              }
           }
        }
          
         /* read byte */
        case CMD_READ_8:
          BDM08_CMD_READB((unsigned int)256*command_buffer[3]+command_buffer[2],command_buffer+1);
          return(2);
        /* read registers */
        case CMD_READ_REGS:
        
          BDM08_CMD_READ_PC(command_buffer+2);  
          command_buffer[1] = command_buffer[3];  /* swap bytes around for little endian */
          
					BDM08_CMD_READ_SP(command_buffer+4);	 //S08
          command_buffer[3] = command_buffer[5];  /* swap bytes around for little endian */
          
					BDM08_CMD_READ_HX(command_buffer+6);
          command_buffer[5] = command_buffer[7];  /* swap bytes around for little endian */
          
					BDM08_CMD_READ_A(command_buffer+7);
          
					BDM08_CMD_READ_CCR(command_buffer+8);					
          return(13);
        /* read block of data from memory */
        case CMD_READ_BLOCK1: 
          {
            /* NOTE: This command returns block of BYTES, endianism does not matter here!!! */
            unsigned char count = command_buffer[4];
            unsigned char count_orig = command_buffer[4];
            unsigned int addr = (unsigned int)256*command_buffer[3]+command_buffer[2];
            unsigned char *data_ptr = command_buffer+1;
             
            if (count>COMMAND_BUFFER_SIZE ) {
              command_buffer[0] = CMD_FAILED; /* requested block is too long to fit into the buffer */
              return(1);  
            }
            if (addr&0x0001) {
              /* start address is odd, fetch the first byte separately */
              BDM08_CMD_READB(addr,data_ptr);  
              addr++;
              data_ptr++;
              count--;
            }
            while(count&0xfe) {
              BDM08_CMD_READB(addr,data_ptr);    /* fetch a word */
              addr+=1;                           /* increment address */
              data_ptr+=1;                       /* increment pointer */
              count-=1;                          /* decrement count */
            }
            if (count) {
              /* there is one extra byte to read at the end of the block */  
              BDM08_CMD_READB(addr,data_ptr);  
            }
            return(count_orig+1);
          }
        /* write byte */
        case CMD_WRITE_8:
          BDM08_CMD_WRITEB((unsigned int)256*command_buffer[3]+command_buffer[2],command_buffer[4]);
          return(1);
       //////////////////////////////////////////// 
        case TCMD_NEXT_DLDATA: {
          
          
           if (state_global == 4){
					 
					   command_buffer[0] = CMD_FAILED 	;
			       return (0);	}
					if (state_global != 2){
					 
					   command_buffer[0] = CMD_FAILED 	;
			       return (0);	}
			       
					else if(state_global ==2){
					    
					    command_buffer[0] = CMD_OK;
					    return (0);
					}
					
					command_buffer[0] = CMD_FAILED;
					return(0) ;
        }
        /////////////////////////////////////////////////
        
        
        /* write block of bytes to memory */
        case CMD_WRITE_BLOCK1: 
          {
            unsigned char count = command_buffer[4];
            unsigned int addr = (unsigned int)256*command_buffer[3]+command_buffer[2];
            unsigned char *data_ptr = command_buffer+5;
        
            if (state_global ==2){
        
              bcommand_buffer[2] = command_buffer[2];
              bcommand_buffer[3] = command_buffer[3] ;
              bcommand_buffer[4] = command_buffer[4];
              bcommand_buffer[5] = command_buffer[5];
              bcommand_buffer[6] = command_buffer[6]; 
              bcommand_buffer[7] = command_buffer[7];
              bcommand_buffer[8] = command_buffer[8] ;
              bcommand_buffer[9] = command_buffer[9];
              bcommand_buffer[10] = command_buffer[10];
              bcommand_buffer[11] = command_buffer[11]; 
              bcommand_buffer[12] = command_buffer[12];
              bcommand_buffer[13] = command_buffer[13] ;
              bcommand_buffer[14] = command_buffer[14];
              bcommand_buffer[15] = command_buffer[15];
              bcommand_buffer[16] = command_buffer[16]; 
              bcommand_buffer[17] = command_buffer[17];
              bcommand_buffer[18] = command_buffer[18] ;
              bcommand_buffer[19] = command_buffer[19];
              bcommand_buffer[20] = command_buffer[20];
              bcommand_buffer[21] = command_buffer[21]; 
              bcommand_buffer[22] = command_buffer[22];
              bcommand_buffer[23] = command_buffer[23] ;
              bcommand_buffer[24] = command_buffer[24];
              bcommand_buffer[25] = command_buffer[25];
              bcommand_buffer[26] = command_buffer[26]; 
              bcommand_buffer[27] = command_buffer[27];
              bcommand_buffer[28] = command_buffer[28] ;
              bcommand_buffer[29] = command_buffer[29];
              bcommand_buffer[30] = command_buffer[30];
              bcommand_buffer[31] = command_buffer[31]; 
              bcommand_buffer[32] = command_buffer[32];
              bcommand_buffer[33] = command_buffer[33] ;
              bcommand_buffer[34] = command_buffer[34];
              bcommand_buffer[35] = command_buffer[35];
              bcommand_buffer[36] = command_buffer[36]; 
              bcommand_buffer[37] = command_buffer[37];
              bcommand_buffer[38] = command_buffer[38] ;
              bcommand_buffer[39] = command_buffer[39];
              bcommand_buffer[40] = command_buffer[40];
              bcommand_buffer[41] = command_buffer[41];
              bcommand_buffer[42] = command_buffer[42];
              bcommand_buffer[43] = command_buffer[43] ;
              bcommand_buffer[44] = command_buffer[44];
              bcommand_buffer[45] = command_buffer[45];
              bcommand_buffer[46] = command_buffer[46]; 
              bcommand_buffer[47] = command_buffer[47];
              bcommand_buffer[48] = command_buffer[48] ;
              bcommand_buffer[49] = command_buffer[49];
              bcommand_buffer[50] = command_buffer[50];
              bcommand_buffer[51] = command_buffer[51]; 
              bcommand_buffer[52] = command_buffer[52];
              bcommand_buffer[53] = command_buffer[53] ;
              bcommand_buffer[54] = command_buffer[54];
              bcommand_buffer[55] = command_buffer[55];
              bcommand_buffer[56] = command_buffer[56]; 
              bcommand_buffer[57] = command_buffer[57];
              bcommand_buffer[58] = command_buffer[58] ;
              bcommand_buffer[59] = command_buffer[59];
              bcommand_buffer[60] = command_buffer[60];
              bcommand_buffer[61] = command_buffer[61]; 
              bcommand_buffer[62] = command_buffer[62];
              bcommand_buffer[63] = command_buffer[63] ;
              bcommand_buffer[64] = command_buffer[64];
              bcommand_buffer[65] = command_buffer[65];
              bcommand_buffer[66] = command_buffer[66]; 
              bcommand_buffer[67] = command_buffer[67];
              bcommand_buffer[68] = command_buffer[68] ;
            
              state_global = 4;
             
              bdm_write_ram();
              // trimval = 3;
              command_buffer[0] = 0x04 	;
			
			 	      return (0);
            
            } else if (state_global!=2){
              
              if (addr&0x0001) {
                /* start address is odd, write the first byte separately */
                BDM08_CMD_WRITEB(addr,*data_ptr);  
                addr++;
                data_ptr++;
                count--;
              }
              while(count&0xfe) {
                /* while there are at least 2 bytes to fetch */
                 BDM08_CMD_WRITEB(addr,*data_ptr);  
                 addr+=1;                              /* increment address */
                 data_ptr+=1;                          /* increment pointer */
                 count-=1;                             /* decrement count */
              }
              if (count) {
                /* there is one extra byte to write at the end of the block */  
                BDM08_CMD_WRITEB(addr,*data_ptr);  
              }
              return(1);
            }   
          }
          
        /* write to PC */
        case CMD_WRITE_REG_PC:
          BDM08_CMD_WRITE_PC((unsigned int)256*command_buffer[3]+command_buffer[2]);
          return(1);
        /* write to SP */
        case CMD_WRITE_REG_SP:
          BDM08_CMD_WRITE_SP((unsigned int)256*command_buffer[3]+command_buffer[2]);
          return(1);
        /* write to IX */
        case CMD_WRITE_REG_X:
          BDM08_CMD_WRITE_HX((unsigned int)256*command_buffer[3]+command_buffer[2]);
          return(1);
        /* write to B:A */
        case CMD_WRITE_REG_D:
          BDM08_CMD_WRITE_A(command_buffer[2]);
          return(1);
        /* write to CCR */
        case CMD_WRITE_REG_CCR:
					BDM08_CMD_WRITE_CCR(command_buffer[2]); /* CCR is stored in little endian notation */
          return(1);
        case CMD_WRITE_BKPT:
					BDM08_CMD_WRITE_BKPT((unsigned int)256*command_buffer[3]+command_buffer[2]); 
					return(1);
        case CMD_READ_BKPT:
					BDM08_CMD_READ_BKPT(command_buffer+2);	
					command_buffer[1] = command_buffer[3]; 
          return(3);
          
       
			 case CMD_FLASH_DLEND:
			 	if (state_global == 2){
			 	
			 	   BDM08_CMD_READB(0x3ffa,&state_global);
			 	   if(state_global == 0xff){
			 	      bdm_programtrim();}
			 	        //VPP OFF
          PTD_PTD4 = 1;
			 	  bdm_rs08softreset(0);
			 	state_global=0;
			 	return(1);
			 	} else{  command_buffer[0] = CMD_FAILED 	;
			
			 	return (1);
			 	}
										
        
        default:    /* unknown command */
          command_buffer[0] = CMD_UNKNOWN;
          return(1);    
      }
    }
  }

    command_buffer[0] = CMD_UNKNOWN;
    return(1);    
  }


