/* header file for opensourcebdm.DLL

Change History:
	 ------------------
	 *****Prominent Notice-This software was modified from TBDML software - 12/05

	 */

#ifndef __OPENSOURCEBDM_H__
#define __OPENSOURCEBDM_H__

#include <windows.h>

#define SEL_HCS08	1
#define SEL_RS08	1
#define SEL_HC12	0
#define SEL_CFV1	1


typedef enum {	/* type of BDM target */
	HC12=0,
	HCS08=1,
	RS08=2,
	CFV1=3
} target_type_e;

typedef enum { /* type of RS08 derivative */
  KA1=0,
  KA2=1,
  LA8=2
} rs08_derivative_type_e;

typedef enum {	/* type of reset action required */
	SPECIAL_MODE=0,
	NORMAL_MODE=1
} target_mode_e;

typedef enum {	/* state of ACKN feature (supported or not) */
	WAIT=0,
	ACKN=1
} ackn_state_e;

typedef enum {	/* target reset detection state */
	RESET_INACTIVE=0,
	RESET_DETECTED=1
} reset_state_e;

typedef enum {	/* how was the target speed detected */
	NO_CONNECTION=0,
	SYNC=1,
	RESERVED=2,
	MANUAL_SETUP=3
} connection_state_e;

typedef struct { /* state of BDM communication */
	ackn_state_e ackn_state;
	reset_state_e reset_state;
	connection_state_e connection_state;
} bdm_status_t;

typedef union {
	struct {
		unsigned int pc;
		unsigned int sp;
		unsigned int ix;
		unsigned int iy;
		unsigned int d;
		unsigned int ccr;
	} hc12;
	struct {
		unsigned int pc;
		unsigned int sp;
		unsigned int hx;
		unsigned int a;
		unsigned int ccr;
	} hcs08;
	struct {
		unsigned int pc;
		unsigned int spc;
		unsigned int x;
		unsigned int a;
		unsigned int ccr;

	} rs08;
} registers_t;


 __declspec(dllexport) unsigned char WINAPI opensourcebdm_dll_version(void); 				/* returns DLL version in BCD */
 __declspec(dllexport) unsigned char WINAPI opensourcebdm_init(void);						/* initialises USB and returns number of devices found */
 __declspec(dllexport) unsigned char WINAPI opensourcebdm_open(unsigned char device_no); 	/* opens a device with given number (0...), returns 0 on success and 1 on error */
 __declspec(dllexport) void			 WINAPI opensourcebdm_close(void);								/* closes currently open device */
 __declspec(dllexport) unsigned int  WINAPI opensourcebdm_get_version(void);				/* returns version of HW (MSB) and SW (LSB) of the interface in BCD format */
 __declspec(dllexport) unsigned char WINAPI opensourcebdm_get_last_sts(void);				/* returns status of the last command: 0 on success and non-zero on failure */
 __declspec(dllexport) unsigned char WINAPI opensourcebdm_set_target_type(target_type_e target_type); /* sets target MCU type, returns 0 on success and non-zero on failure */
 __declspec(dllexport) unsigned char WINAPI opensourcebdm_set_derivative_type(unsigned char derivative_type);	/* sets target derivative type, returns 0 on success and non-zero on failure */
 __declspec(dllexport) unsigned char WINAPI opensourcebdm_target_sync(void); 				/* measures BDM frequency of the target using the SYNC feature and connects to it, returns 0 on success and non-zero on failure */
 __declspec(dllexport) unsigned char WINAPI opensourcebdm_target_reset(target_mode_e target_mode); /* resets the target to normal or special mode, returns 0 on success and non-zero on failure */
 __declspec(dllexport) unsigned char WINAPI opensourcebdm_bdm_sts(bdm_status_t *bdm_status); /* fills user supplied structure with actual state of BDM communication channel, returns 0 on success and non-zero on failure */


 __declspec(dllexport) unsigned char WINAPI opensourcebdm_target_go(void); 				/* starts target execution from current PC address, returns 0 on success and non-zero on failure */
 __declspec(dllexport) unsigned char WINAPI opensourcebdm_target_step(void); 				/* steps over a single target instruction, returns 0 on success and non-zero on failure */
 __declspec(dllexport) unsigned char WINAPI opensourcebdm_target_halt(void); 				/* brings the target into active background mode, returns 0 on success and non-zero on failure */
 __declspec(dllexport) unsigned char WINAPI opensourcebdm_set_speed(float crystal_frequency); /* sets the communication speed, parameter is crystal frequency in MHz, returns 0 on success and non-zero on failure */
 __declspec(dllexport) float WINAPI opensourcebdm_get_speed(void); 						/* returns crystal frequency of the target in MHz */

 
#if SEL_CFV1
	__declspec(dllexport) BOOL WINAPI opensourcebdm_write_mem(unsigned int address, unsigned int numBytes, unsigned int accessType, unsigned char *buffer);
	__declspec(dllexport) BOOL WINAPI opensourcebdm_read_mem(unsigned int address, unsigned int numBytes, unsigned int accessType, unsigned char *buffer); 
	__declspec(dllexport) unsigned int WINAPI  opensourcebdm_read_reg(unsigned char crn);
	__declspec(dllexport) unsigned int WINAPI  opensourcebdm_read_creg(unsigned char crn);
	__declspec(dllexport) unsigned int WINAPI  opensourcebdm_read_dreg(unsigned char crn);
	__declspec(dllexport) unsigned char WINAPI  opensourcebdm_read_xcsr_byte(void);
	__declspec(dllexport) unsigned char WINAPI  opensourcebdm_read_csr2_byte(void);
	__declspec(dllexport) unsigned char WINAPI  opensourcebdm_read_csr3_byte(void);
	__declspec(dllexport) void WINAPI  opensourcebdm_write_xcsr_byte(unsigned char value);
	__declspec(dllexport) void WINAPI  opensourcebdm_write_csr2_byte(unsigned char value);
	__declspec(dllexport) void WINAPI  opensourcebdm_write_csr3_byte(unsigned char value);
	__declspec(dllexport) void WINAPI  opensourcebdm_write_reg(unsigned char crn, unsigned int value);
	__declspec(dllexport) void WINAPI  opensourcebdm_write_creg(unsigned char crn, unsigned int value);
	__declspec(dllexport) void WINAPI  opensourcebdm_write_dreg(unsigned char crn, unsigned int value);
#endif

 
#if SEL_HCS08 || SEL_RS08 || SEL_HCS12
	__declspec(dllexport) unsigned char WINAPI opensourcebdm_read_byte(unsigned int address); /* reads one byte from memory */
	__declspec(dllexport) void WINAPI opensourcebdm_write_byte(unsigned int address, unsigned char data); /* writes one byte to memory */
	__declspec(dllexport) void WINAPI opensourcebdm_read_block(unsigned int address, unsigned int count, unsigned char *data); /* reads block from memory, data is returned to the user supplied buffer */
	__declspec(dllexport) void WINAPI opensourcebdm_write_block(unsigned int address, unsigned int count, unsigned char *data); /* writes block to memory */
	__declspec(dllexport) unsigned char WINAPI opensourcebdm_read_regs(registers_t *registers); /* reads contents of core registers, returns 0 on success and non-zero on failure */
	__declspec(dllexport) void WINAPI opensourcebdm_write_reg_pc(unsigned int value); 		/* writes PC */
	__declspec(dllexport) void WINAPI opensourcebdm_write_reg_sp(unsigned int value); 		/* writes SP */
	__declspec(dllexport) void WINAPI opensourcebdm_write_reg_x(unsigned int value);	 		/* writes H:X (HCS08) or IX (HC12) */
	__declspec(dllexport) void WINAPI opensourcebdm_write_reg_y(unsigned int value); 			/* writes IY (HC12 only) */
	__declspec(dllexport) void WINAPI opensourcebdm_write_reg_d(unsigned int value); 			/* writes B:A (HC12) or A (HCS08) */
	__declspec(dllexport) void WINAPI opensourcebdm_write_reg_ccr(unsigned int value); 		/* writes CCR */

#endif

#if SEL_HCS08 || SEL_RS08 
  	 __declspec(dllexport) unsigned char WINAPI opensourcebdm_read_status(void);
	 __declspec(dllexport) void WINAPI opensourcebdm_write_bkpt(unsigned int value);
	 __declspec(dllexport) void WINAPI opensourcebdm_write_control(unsigned char value);
	 __declspec(dllexport) unsigned int WINAPI opensourcebdm_read_bkpt(void);

	//ADD for RS08
	 __declspec(dllexport) void WINAPI opensourcebdm_mem_dlstart(void);
	 __declspec(dllexport) void WINAPI opensourcebdm_mem_dlend(void);

#endif

#if SEL_HCS12
	 __declspec(dllexport) unsigned char WINAPI opensourcebdm_read_bd(unsigned int address); 	/* reads one byte from the BDM area of memory */
	 __declspec(dllexport) unsigned char WINAPI opensourcebdm_write_bd(unsigned int address, unsigned char data); /* writes one byte to the BDM area of memory, returns 0 on success and non-zero on failure */
	 __declspec(dllexport) unsigned int WINAPI opensourcebdm_read_word(unsigned int address); 	/* reads one word from memory (address must be aligned) */
	 __declspec(dllexport) void WINAPI opensourcebdm_write_word(unsigned int address, unsigned int data); /* writes one word to memory (address must be aligned) */
#endif



#endif __OPENSOURCEBDM_H__
