/*
    Open Source BDM - interface DLL
    Copyright (C) 2005

	 Change History:
	 ------------------
	 *****Prominent Notice-This software was modified from TBDML software - 12/05

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <windows.h>
#include <time.h>
#include <stdio.h>
#include "log.h"
#include "version.h"
#include "opensourcebdm_usb.h"
#include "commands.h"
#include "opensourcebdm.h"
#include "stdafx.h"

/* define symbol "LOG" during compilation to produce a log file opensourcebdm_dll.log */

char MEMDLFLAG2;
char MEMDLFLAG3;
char RS08FLAG;
char FLASHDLFLAG2;

char RS08_KA1FLAG;
char RS08_KA2FLAG;
char RS08_LA8FLAG;

static unsigned char usb_data[MAX_DATA_SIZE+2];  

/* returns version of the DLL in BCD format */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_dll_version(void) {
	return(opensourcebdm_DLL_VERSION);
}

/* initialises USB and returns number of devices found */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_init(void) {
	unsigned char i;
	opensourcebdm_usb_find_devices();	/* look for devices on all USB busses */
	i=opensourcebdm_usb_cnt();
	print("opensourcebdm_INIT: Usb initialised, found %d device(s)\r\n",i);
	return(i);					/* count the devices found and return the number */
}

/* opens a device with given number (0...) */
/* returns 0 on success and 1 on error */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_open(unsigned char device_no) {
	print("opensourcebdm_OPEN: Trying to open device #%d\r\n",device_no);
	return(opensourcebdm_usb_open(device_no));
}

/* closes currently open device */
__declspec(dllexport)void WINAPI opensourcebdm_close(void) {
	print("opensourcebdm_CLOSE: Trying to close the device\r\n");
	opensourcebdm_usb_close();
}

/* gets version of the interface (HW and SW) in BCD format */
__declspec(dllexport) unsigned int WINAPI opensourcebdm_get_version(void) {
	usb_data[0]=3;	/* get 3 bytes */
	usb_data[1]=CMD_GET_VER;
	opensourcebdm_usb_recv_ep0(usb_data);
	return((*((unsigned int *)(usb_data+1)))&0xffff);
}

/* returns status of the last command (either the command number or CMD_FAILED or CMD_UNKNOWN */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_get_last_sts(void) {
	unsigned char i;
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_GET_LAST_STATUS;
	opensourcebdm_usb_recv_ep0(usb_data);
	if ((usb_data[0]==CMD_FAILED)||(usb_data[0]==CMD_UNKNOWN)) i=1; else i=0;
	return(i);
}


/* sets target MCU type */
/* returns 0 on success and non-zero on failure */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_set_target_type (target_type_e target_type) {
	usb_data[0]=1;	 /* get 1 byte */ 

	usb_data[1]=CMD_SET_TARGET;
	RS08FLAG = 0;
	if (target_type == RS08)
	{
			//print("RS08 found, RS08FLAG = 1...%d... \r\n", target_type);
			RS08FLAG = 1;

	} //else  print("not RS08 found, RS08FLAG = 0...%d... \r\n", target_type);

	usb_data[2]=target_type;
	opensourcebdm_usb_recv_ep0(usb_data);
	return(!(usb_data[0]==CMD_SET_TARGET));
}

/* sets target derivative type */
/* returns 0 on success and non-zero on failure */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_set_derivative_type(unsigned char derivative_type) {
	usb_data[0]=1;	 /* get 1 byte */ 

	RS08_KA1FLAG = 0;
	RS08_KA2FLAG = 0;
	RS08_LA8FLAG = 0;

	switch (derivative_type) {
	case KA1:
		RS08_KA1FLAG =1; break;
	case KA2:
		RS08_KA2FLAG =1; break;
	case LA8:
		RS08_LA8FLAG =1; break;
	default:
		RS08_KA2FLAG =1; break;
	}

	usb_data[1]=CMD_SET_DERIVATIVE;
	usb_data[2]=derivative_type;
	opensourcebdm_usb_recv_ep0(usb_data);
	return(!(usb_data[0]==CMD_SET_DERIVATIVE));
}

/* measures BDM frequency of the target using the SYNC feature and connects to it */
/* returns 0 on success and non-zero on failure */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_target_sync(void) {
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_CONNECT;
	opensourcebdm_usb_recv_ep0(usb_data);
	return(!(usb_data[0]==CMD_CONNECT));
}

/* resets the target to normal or special mode */
/* returns 0 on success and non-zero on failure */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_target_reset(target_mode_e target_mode) {
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_RESET;
	usb_data[2]=target_mode;
	opensourcebdm_usb_recv_ep0(usb_data);
	return(!(usb_data[0]==CMD_RESET));
}

/* fills user supplied structure with actual state of BDM communication channel */
/* returns 0 on success and non-zero on failure */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_bdm_sts(bdm_status_t *bdm_status) {
	usb_data[0]=3;	 /* get 3 bytes */
	usb_data[1]=CMD_GET_STATUS;
	opensourcebdm_usb_recv_ep0(usb_data);
	if (usb_data[0]!=CMD_GET_STATUS) return(1);
	bdm_status->ackn_state = (ackn_state_e)(usb_data[1]&0x01);
	bdm_status->reset_state = (reset_state_e)((usb_data[1]&0x02)>>1);
	bdm_status->connection_state = (connection_state_e)((usb_data[1]&0x18)>>3);
	return(0);
}



/* starts target execution from current PC address */
/* returns 0 on success and non-zero on failure */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_target_go(void) {
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_GO1;
	opensourcebdm_usb_recv_ep0(usb_data);
	return(!(usb_data[0]==CMD_GO1));
}

/* steps over a single target instruction */
/* returns 0 on success and non-zero on failure */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_target_step(void) {
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_STEP1;
	opensourcebdm_usb_recv_ep0(usb_data);
	return(!(usb_data[0]==CMD_STEP1));
}

/* brings the target into active background mode */
/* returns 0 on success and non-zero on failure */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_target_halt(void) {
	usb_data[0]=1;	 /* get 1 byte */ 
	usb_data[1]=CMD_HALT;
	opensourcebdm_usb_recv_ep0(usb_data);
	return(!(usb_data[0]==CMD_HALT));
}

/* returns crystal frequency of the target in MHz */
__declspec(dllexport) float WINAPI  opensourcebdm_get_speed(void) {
	usb_data[0]=3;	 /* get 3 bytes */ 
	usb_data[1]=CMD_READ_SPEED1;
	opensourcebdm_usb_recv_ep0(usb_data);
	return (float)(60.0 * 2 * 128 / (*((unsigned int *)(usb_data+1))&0xffff));
}

/* sets the communication speed, parameter is crystal frequency in MHz */
/* returns 0 on success and non-zero on failure */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_set_speed(float crystal_frequency) {
	/* the command parameter in this case is expected length of SYNC pulse expressed in 60MHz ticks */

	usb_data[0]=1;	 /* get 1 byte */  
	usb_data[1]=CMD_SET_SPEED1;
	*((unsigned int*)(usb_data+2)) = (unsigned int) (60.0 * 2 * 128 / crystal_frequency);
	opensourcebdm_usb_recv_ep0(usb_data);
	return(!(usb_data[0]==CMD_SET_SPEED1));
}

#if SEL_HCS08 || SEL_RS08 

/* returns status of the last command (either the command number or CMD_FAILED or CMD_UNKNOWN */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_target_readyfor_datald(void) {
	unsigned char i;
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=TCMD_NEXT_DLDATA;
	print("PRECHECK OF TARGET BEFORE NEXT DATA\r\n");
	opensourcebdm_usb_recv_ep0(usb_data);
	if ((usb_data[0]==CMD_FAILED)||(usb_data[0]==CMD_UNKNOWN)) i=1; else i=0;
	return(i);
}


///////////////////***************************************
unsigned char /*WINAPI __declspec(dllexport)*/opensourcebdm_flash_dlstart(void){
	//
unsigned char i;

	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_FLASH_DLSTART;
	print("CMD_FLASH_DLSTART...... \r\n");
	opensourcebdm_usb_recv_ep0(usb_data);
	if ((usb_data[0]==CMD_FAILED)||(usb_data[0]==CMD_UNKNOWN)) i=1; else i=0;
	return(i);
}

///////////////////***************************************
unsigned char /*WINAPI __declspec(dllexport)*/ opensourcebdm_flash_dlend(void) {
	//
	unsigned char i;

	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_FLASH_DLEND;
	print("CMD_FLASH_DLEND......  \r\n");
	opensourcebdm_usb_recv_ep0(usb_data);
	if ((usb_data[0]==CMD_FAILED)||(usb_data[0]==CMD_UNKNOWN)) i=1; else i=0;
	return(i);
}


////////////////////////////************************************

//********************************
__declspec(dllexport) void WINAPI opensourcebdm_mem_dlstart(void){
	////nsigned char i;
MEMDLFLAG2 = 1;
MEMDLFLAG3 = 1;
}


__declspec(dllexport) void WINAPI  opensourcebdm_mem_dlend(void) {
	MEMDLFLAG2 = 0;
	MEMDLFLAG3 = 0;
	opensourcebdm_flash_dlend();
}


#endif  

/*********************************************************************************************/
/* Memory Access			                                                                 */
/*********************************************************************************************/
#if SEL_CFV1
// CFV1 supports BYTE(8 bit), WORD(16 bit) and LONG WORD(32 bit) memory accesses.
// The address size for memory access is 24 bit.

#define MAX_DATA_SIZE_CFV1         96

/* Return '1' on success. Return '0' on failure.*/
/* The input parameter "address" should be aligned with 0-modulo-access size. Otherwise, the hardware will force the alignments. */
__declspec(dllexport) 
BOOL WINAPI opensourcebdm_read_mem(unsigned int address, unsigned int numBytes, unsigned int accessType, unsigned char *buffer) 
{
	unsigned int i;

	while (numBytes> MAX_DATA_SIZE_CFV1) {
		usb_data[0]= (MAX_DATA_SIZE_CFV1&0xfe)+1; //(number of data + one command) which is expected to receive
		usb_data[1]= CMD_READ_BLOCK_WITH_ACCESSTYPE;
		usb_data[2]= (address >>0)  & 0x000000ff;
		usb_data[3]= (address >>8)  & 0x000000ff;
		usb_data[4]= (address >>16) & 0x000000ff;
		usb_data[5]= (unsigned char) accessType;
		if (opensourcebdm_usb_recv_ep0(usb_data)) 
			return(0);

		for (i=0;i<(MAX_DATA_SIZE_CFV1&0xfe);i++) 
			*(buffer++) = usb_data[i+1];
		address += (MAX_DATA_SIZE_CFV1&0xfe);	/* update address */
		numBytes -= (MAX_DATA_SIZE_CFV1&0xfe);	/* update count */
	}

	if (numBytes) {
		usb_data[0]= numBytes+1;
		usb_data[1]=CMD_READ_BLOCK_WITH_ACCESSTYPE;
		usb_data[2]= (address >>0)  & 0x000000ff;
		usb_data[3]= (address >>8)  & 0x000000ff;
		usb_data[4]= (address >>16) & 0x000000ff;
		usb_data[5]= (unsigned char) accessType;

		if (opensourcebdm_usb_recv_ep0(usb_data))
			return(0);

		for (i=0;i<numBytes;i++) 
			*(buffer++) = usb_data[i+1];
	}
     
	return(1);
}

/* Return 'true' on success. Return 'false' on failure.*/
/* The input parameter "address" should be aligned with 0-modulo-access size. Otherwise, the hardware will force the alignments. */
/* numBytes is multiple of accessType. */
__declspec(dllexport) 
BOOL WINAPI opensourcebdm_write_mem(unsigned int address, unsigned int numBytes, unsigned int accessType, unsigned char *buffer) {

	unsigned int i;

	while (numBytes>MAX_DATA_SIZE_CFV1) {
		usb_data[0]= (MAX_DATA_SIZE_CFV1&0xfe)+5;
		usb_data[1]= CMD_WRITE_BLOCK_WITH_ACCESSTYPE;

		// address is little endian
		usb_data[2]= (address >>0)  & 0x000000ff;
		usb_data[3]= (address >>8)  & 0x000000ff;
		usb_data[4]= (address >>16) & 0x000000ff;
		usb_data[5]= (unsigned char) accessType;

		// copy the data buffer to usb buffer in order (raw data in big endian)
		for (i=0;i<(MAX_DATA_SIZE_CFV1&0xfe);i++) 
			usb_data[i+6] = *(buffer++);

		if (opensourcebdm_usb_send_ep0(usb_data)) 
			return(0);

		address += (MAX_DATA_SIZE_CFV1&0xfe);	/* update address */
		numBytes -= (MAX_DATA_SIZE_CFV1&0xfe);	/* update count */
	}

	if (numBytes) {
		usb_data[0]= numBytes+5;
		usb_data[1]= CMD_WRITE_BLOCK_WITH_ACCESSTYPE;
		usb_data[2]= (address >>0)  & 0x000000ff;
		usb_data[3]= (address >>8)  & 0x000000ff;
		usb_data[4]= (address >>16) & 0x000000ff;
		usb_data[5]= (unsigned char) accessType;

		for (i=0;i<numBytes;i++) 
			usb_data[i+6] = *(buffer++);

		if (opensourcebdm_usb_send_ep0(usb_data))
			return(0);

	}
	return(1);
}
#endif


#if SEL_HCS08 || SEL_RS08 || SEL_HCS12
// CFV1 supports BYTE(8 bit), WORD(16 bit) and LONG WORD(32 bit) memory accesses.
// The address size for memory access is 24 bit.
// Here, supported address size is 16 bit.

/* reads one byte from memory */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_read_byte(unsigned int address) {
	usb_data[0]=2;	 /* get 2 bytes */
	usb_data[1]=CMD_READ_8;
	*((unsigned int*)(usb_data+2)) = address;
	opensourcebdm_usb_recv_ep0(usb_data);
	return(usb_data[1]);
}

/* writes one byte to memory */
__declspec(dllexport) void WINAPI opensourcebdm_write_byte(unsigned int address, unsigned char data) {
	usb_data[0]=4;	 /* send 4 bytes */
	usb_data[1]=CMD_WRITE_8;
	*((unsigned int*)(usb_data+2)) = address;
	usb_data[4] = data;
	opensourcebdm_usb_send_ep0(usb_data);
}
/* reads block from memory */
/* data is returned to the user supplied buffer */
__declspec(dllexport) void WINAPI opensourcebdm_read_block(unsigned int address, unsigned int count, unsigned char *data) {
	unsigned int i;
	if ((count>MAX_DATA_SIZE)&&(address&0x0001)) {
		/* if address is odd and we are going to read more than one block make the first packet 1 byte short to align the address */
		usb_data[1]=CMD_READ_BLOCK1;
		*((unsigned int*)(usb_data+2)) = address;
		usb_data[4]=(MAX_DATA_SIZE&0xfe)-1;
		#ifdef FAST_BLOCK
			usb_data[0]=4;
			opensourcebdm_usb_send_ep2(usb_data);
			usb_data[0]=(MAX_DATA_SIZE&0xfe)-1+1;
			opensourcebdm_usb_recv_ep2(usb_data);
		#else
			usb_data[0]=(MAX_DATA_SIZE&0xfe)-1+1;
			opensourcebdm_usb_recv_ep0(usb_data);
		#endif
		for (i=0;i<(MAX_DATA_SIZE&0xfe)-1;i++) *(data++) = usb_data[i+1];
		address += (MAX_DATA_SIZE&0xfe)-1;	/* update address */
		count -= (MAX_DATA_SIZE&0xfe)-1;	/* update count */
	}
	while (count>MAX_DATA_SIZE) {
		usb_data[1]=CMD_READ_BLOCK1;
		*((unsigned int*)(usb_data+2)) = address;
		usb_data[4]=(MAX_DATA_SIZE&0xfe);
		#ifdef FAST_BLOCK
			usb_data[0]=4;
			opensourcebdm_usb_send_ep2(usb_data);
			usb_data[0]=(MAX_DATA_SIZE&0xfe)+1;
			opensourcebdm_usb_recv_ep2(usb_data);
		#else
			usb_data[0]=(MAX_DATA_SIZE&0xfe)+1;
			opensourcebdm_usb_recv_ep0(usb_data);
		#endif
		for (i=0;i<(MAX_DATA_SIZE&0xfe);i++) *(data++) = usb_data[i+1];
		address += (MAX_DATA_SIZE&0xfe);	/* update address */
		count -= (MAX_DATA_SIZE&0xfe);		/* update count */
	}
	if (count) {
		usb_data[1]=CMD_READ_BLOCK1;
		*((unsigned int*)(usb_data+2)) = address;
		usb_data[4]=count;
		#ifdef FAST_BLOCK
			usb_data[0]=4;
			opensourcebdm_usb_send_ep2(usb_data);
			usb_data[0]=count+1;
			opensourcebdm_usb_recv_ep2(usb_data);
		#else
			usb_data[0]=count+1;
			opensourcebdm_usb_recv_ep0(usb_data);
		#endif
		for (i=0;i<count;i++) *(data++) = usb_data[i+1];
	}
}

/* writes block to memory */
__declspec(dllexport) void WINAPI  opensourcebdm_write_block(unsigned int address, unsigned int count, unsigned char *data) {

	unsigned int i;
	unsigned int flash_lo_addr, flash_hi_addr;
	//print("RS08FLAG = %d... \r\n", RS08FLAG);

	if (RS08FLAG == 1) {	
		if (RS08_KA1FLAG){
			flash_lo_addr = 0x3C00;
			flash_hi_addr = 0x3FFF;
		} 
		else if (RS08_KA2FLAG){
			flash_lo_addr = 0x3800;
			flash_hi_addr = 0x3FFF;
		}
		else if (RS08_LA8FLAG){
			flash_lo_addr = 0x2000;
			flash_hi_addr = 0x3FFF;
		}
		else {
			flash_lo_addr = 0x3C00;
			flash_hi_addr = 0x3FFF;
		} 

		if ((address >= flash_lo_addr) && (address <=flash_hi_addr) && (MEMDLFLAG2 == 1) /*&& (MEMDLFLAG3 == 1) */) {

			 FLASHDLFLAG2 = 1;
			 print("FLASH PROGRAMMING, FLASHDLFLAG2 = 1\r\n");
			 if ((MEMDLFLAG3 == 1)) {
				 print("FIRST BLOCK REQUIRED TARGET TO MASS ERASE BEFORE PROGRAMMING\r\n");
				 opensourcebdm_flash_dlstart();   //Donnie will mass erase
				 MEMDLFLAG3 = 0;
				 if (FLASHDLFLAG2 ==1) { while (opensourcebdm_target_readyfor_datald() == 1);}
			 }

		}
		else {
			FLASHDLFLAG2 = 0;
			print("NON-FLASH PROGRAMMING,FLASHDLFLAG2 = 0\r\n");
		}
	}
	if ((count>MAX_DATA_SIZE)&&(address&0x0001)) {
		/* if address is odd and we are going to write more than one block make the first packet 1 byte short to align the address */
		usb_data[0]=(MAX_DATA_SIZE&0xfe)-1+4;
		usb_data[1]=CMD_WRITE_BLOCK1;
		*((unsigned int*)(usb_data+2)) = address;
		usb_data[4]=(MAX_DATA_SIZE&0xfe)-1;
		for (i=0;i<(MAX_DATA_SIZE&0xfe)-1;i++) usb_data[i+5] = *(data++);
		#ifdef FAST_BLOCK
			opensourcebdm_usb_send_ep2(usb_data);	/* Tx data */
			usb_data[0]=1;					/* read status */
			opensourcebdm_usb_recv_ep2(usb_data);
		#else
			opensourcebdm_usb_send_ep0(usb_data);
		#endif
		address += (MAX_DATA_SIZE&0xfe)-1;	/* update address */
		count -= (MAX_DATA_SIZE&0xfe)-1;	/* update count */
		
		if (RS08FLAG == 1) {	
			if (FLASHDLFLAG2 ==1) { while (opensourcebdm_target_readyfor_datald() == 1);}
		}
	}


	while (count>MAX_DATA_SIZE) {
		usb_data[0]=(MAX_DATA_SIZE&0xfe)+4;
		usb_data[1]=CMD_WRITE_BLOCK1;
		*((unsigned int*)(usb_data+2)) = address;
		usb_data[4]=(MAX_DATA_SIZE&0xfe);
		for (i=0;i<(MAX_DATA_SIZE&0xfe);i++) usb_data[i+5] = *(data++);
		#ifdef FAST_BLOCK
			opensourcebdm_usb_send_ep2(usb_data);	/* Tx data */
			usb_data[0]=1;					/* read status */
			opensourcebdm_usb_recv_ep2(usb_data);
		#else
			opensourcebdm_usb_send_ep0(usb_data);
		#endif
		address += (MAX_DATA_SIZE&0xfe);	/* update address */
		count -= (MAX_DATA_SIZE&0xfe);		/* update count */

		if (RS08FLAG == 1) {	
			if (FLASHDLFLAG2 ==1) { while (opensourcebdm_target_readyfor_datald() == 1);}
		}
	}

	if (count) {
		usb_data[0]=count+4;
		usb_data[1]=CMD_WRITE_BLOCK1;
		*((unsigned int*)(usb_data+2)) = address;
		usb_data[4]=count;
		for (i=0;i<count;i++) usb_data[i+5] = *(data++);
		#ifdef FAST_BLOCK
			opensourcebdm_usb_send_ep2(usb_data);	/* Tx data */
			usb_data[0]=1;					/* read status */
			opensourcebdm_usb_recv_ep2(usb_data);
		#else
			opensourcebdm_usb_send_ep0(usb_data);
		#endif

		if (RS08FLAG == 1) {	
			if (FLASHDLFLAG2 ==1) { while (opensourcebdm_target_readyfor_datald() == 1);}
		}
	}
}
#endif


#if SEL_HCS12
// no WORD operations for HCS08
// The address size for memory access of CFV1 is 24 bit.
// Here, the address size is 16 bit. 

/* reads one word from memory (address must be aligned) */
__declspec(dllexport) unsigned int WINAPI opensourcebdm_read_word(unsigned int address) {
	usb_data[0]=3;	 /* get 3 bytes */
	usb_data[1]=CMD_READ_16;
	*((unsigned int*)(usb_data+2)) = address;
	opensourcebdm_usb_recv_ep0(usb_data);
	return((*((unsigned int *)(usb_data+1))&0xffff));
}

/* writes one word to memory (address must be aligned) */
__declspec(dllexport) void WINAPI opensourcebdm_write_word(unsigned int address, unsigned int data) {
	usb_data[0]=5;	 /* send 5 bytes */
	usb_data[1]=CMD_WRITE_16;
	*((unsigned int*)(usb_data+2)) = address;
	*((unsigned int*)(usb_data+4)) = data;
	opensourcebdm_usb_send_ep0(usb_data);
}

/* reads one byte from the BDM area of memory */
/* returns byte read from memory */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_read_bd(unsigned int address) {
	//ST======================================= ***********************
	{
	usb_data[0]=2;	 /* get 2 bytes */
	usb_data[1]=CMD_READ_BD;
	*((unsigned int*)(usb_data+2)) = address;
	opensourcebdm_usb_recv_ep0(usb_data);
	return(usb_data[1]);
	}
}

/* writes one byte to the BDM area of memory */
/* returns 0 on success and non-zero on failure */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_write_bd(unsigned int address, unsigned char data) {
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_WRITE_BD;
	*((unsigned int*)(usb_data+2)) = address;
	usb_data[4] = data;
	opensourcebdm_usb_recv_ep0(usb_data);
	return(!(usb_data[0]==CMD_WRITE_BD));
}

#endif




/*********************************************************************************************/
/* Register Access			                                                                 */
/*********************************************************************************************/

/************************/
/* CFV1 Register Access */
/************************/

#if SEL_CFV1

__declspec(dllexport) unsigned int WINAPI  opensourcebdm_read_reg(unsigned char crn) {

	unsigned int longWordVal;
	unsigned char tmp[4];

	usb_data[0]=5;
	usb_data[1]=CMD_READ_REG_32;
	usb_data[2]=crn;
	opensourcebdm_usb_recv_ep0(usb_data);

	tmp[0] = usb_data[1];
	tmp[1] = usb_data[2];
	tmp[2] = usb_data[3];
	tmp[3] = usb_data[4];

	longWordVal = *(unsigned int*)((void*)(tmp));
	return(longWordVal); 
}

__declspec(dllexport) unsigned int WINAPI  opensourcebdm_read_creg(unsigned char crn) {
	unsigned int longWordVal;
	unsigned char tmp[4];

	usb_data[0]=5;
	usb_data[1]=CMD_READ_CREG_32;
	usb_data[2]=crn;
	opensourcebdm_usb_recv_ep0(usb_data);

	tmp[0] = usb_data[1];
	tmp[1] = usb_data[2];
	tmp[2] = usb_data[3];
	tmp[3] = usb_data[4];

	longWordVal = *(unsigned int*)((void*)(tmp));
	return(longWordVal); 
}

__declspec(dllexport) unsigned int WINAPI  opensourcebdm_read_dreg(unsigned char crn) {
	unsigned int longWordVal;
	unsigned char tmp[4];

	usb_data[0]=5;
	usb_data[1]=CMD_READ_DREG_32;
	usb_data[2]=crn;
	opensourcebdm_usb_recv_ep0(usb_data);

	tmp[0] = usb_data[1];
	tmp[1] = usb_data[2];
	tmp[2] = usb_data[3];
	tmp[3] = usb_data[4];

	longWordVal = *(unsigned int*)((void*)(tmp));
	return(longWordVal); 
}

__declspec(dllexport) unsigned char WINAPI  opensourcebdm_read_xcsr_byte(void) {
	usb_data[0]=2;
	usb_data[1]=CMD_READ_XCSR_8;
	opensourcebdm_usb_recv_ep0(usb_data);
	return((unsigned char)usb_data[1]);
}

__declspec(dllexport) unsigned char WINAPI  opensourcebdm_read_csr2_byte(void) {
	usb_data[0]=2;
	usb_data[1]=CMD_READ_CSR2_8;
	opensourcebdm_usb_send_ep0(usb_data);
	return((unsigned char)usb_data[1]);
}

__declspec(dllexport) unsigned char WINAPI  opensourcebdm_read_csr3_byte(void) {
	usb_data[0]=2;
	usb_data[1]=CMD_READ_CSR3_8;
	opensourcebdm_usb_recv_ep0(usb_data);
	return((unsigned char)usb_data[1]);
}

__declspec(dllexport) void WINAPI  opensourcebdm_write_xcsr_byte(unsigned char value) {
	usb_data[0]=2;
	usb_data[1]=CMD_WRITE_XCSR_8;
	usb_data[2] = value;
	opensourcebdm_usb_send_ep0(usb_data);
}


__declspec(dllexport) void WINAPI  opensourcebdm_write_csr2_byte(unsigned char value) {
	usb_data[0]=2;
	usb_data[1]=CMD_WRITE_CSR2_8;
	usb_data[2] = value;
	opensourcebdm_usb_send_ep0(usb_data);
}

__declspec(dllexport) void WINAPI  opensourcebdm_write_csr3_byte(unsigned char value) {
	usb_data[0]=2;
	usb_data[1]=CMD_WRITE_CSR3_8;
	usb_data[2] = value;
	opensourcebdm_usb_send_ep0(usb_data);
}


__declspec(dllexport) void WINAPI  opensourcebdm_write_reg(unsigned char crn, unsigned int value) {
	usb_data[0]=6;
	usb_data[1]=CMD_WRITE_REG_32;
	usb_data[2]=crn;
	*((unsigned int*)(void *)(usb_data+3)) = value;
	opensourcebdm_usb_send_ep0(usb_data);
}

__declspec(dllexport) void WINAPI  opensourcebdm_write_creg(unsigned char crn, unsigned int value) {
	usb_data[0]=6;
	usb_data[1]=CMD_WRITE_CREG_32;
	usb_data[2]=crn;
	*((unsigned int*)(usb_data+3)) = value;
	opensourcebdm_usb_send_ep0(usb_data);
}

__declspec(dllexport) void WINAPI  opensourcebdm_write_dreg(unsigned char crn, unsigned int value) {
	usb_data[0]=6;
	usb_data[1]=CMD_WRITE_DREG_32;
	usb_data[2]=crn;
	*((unsigned int*)(usb_data+3)) = value;
	opensourcebdm_usb_send_ep0(usb_data);
}
#endif 


/************************************/
/* HCS08/RS08/HCS12 Register Access */
/************************************/

/* reads contents of core registers */
/* returns 0 on success and non-zero on failure */
__declspec(dllexport) unsigned char WINAPI opensourcebdm_read_regs(registers_t *registers) {

#if SEL_HCS08 || SEL_RS08

	if (RS08FLAG == 1) {		
		usb_data[0]=9;	 //return 9
		usb_data[1]=CMD_READ_REGS;
		opensourcebdm_usb_recv_ep0(usb_data);
		registers->rs08.pc = (*((unsigned int*)(usb_data+1)))&0xffff;
		registers->rs08.spc = (*((unsigned int*)(usb_data+3)))&0xffff;
		registers->rs08.x = (*((unsigned int*)(usb_data+5)))&0xffff;//not used
		registers->rs08.a = (*((unsigned int*)(usb_data+7)))&0x00ff;
		registers->rs08.ccr = (*((unsigned int*)(usb_data+8)))&0x00ff;//not used
		return(!(usb_data[0]==CMD_READ_REGS));
	}
	else
	{
		usb_data[0]=9;	 //return 9
		usb_data[1]=CMD_READ_REGS;
		opensourcebdm_usb_recv_ep0(usb_data);
		registers->hcs08.pc = (*((unsigned int*)(usb_data+1)))&0xffff;
		registers->hcs08.sp = (*((unsigned int*)(usb_data+3)))&0xffff;
		registers->hcs08.hx = (*((unsigned int*)(usb_data+5)))&0xffff;
		registers->hcs08.a = (*((unsigned int*)(usb_data+7)))&0x00ff;
		registers->hcs08.ccr= (*((unsigned int*)(usb_data+8)))&0x00ff;
		return(!(usb_data[0]==CMD_READ_REGS));
	}
#endif

#if SEL_HCS12

		usb_data[0]=13;	 // return 13 bytes
		usb_data[1]=CMD_READ_REGS;
		opensourcebdm_usb_recv_ep0(usb_data);
		registers->hc12.pc = (*((unsigned int*)(usb_data+1)))&0xffff;
		registers->hc12.sp = (*((unsigned int*)(usb_data+3)))&0xffff;
		registers->hc12.ix = (*((unsigned int*)(usb_data+5)))&0xffff;
		registers->hc12.iy = (*((unsigned int*)(usb_data+7)))&0xffff;
		registers->hc12.d  = (*((unsigned int*)(usb_data+9)))&0xffff;
		registers->hc12.ccr= (*((unsigned int*)(usb_data+11)))&0xffff;
		return(!(usb_data[0]==CMD_READ_REGS));
#endif

}

/* writes PC */
__declspec(dllexport) void WINAPI  opensourcebdm_write_reg_pc(unsigned int value) {
	usb_data[0]=3;	 /* send 3 bytes */
	usb_data[1]=CMD_WRITE_REG_PC;
	*((unsigned int*)(usb_data+2)) = value;
	opensourcebdm_usb_send_ep0(usb_data);
}

/* writes SP */
__declspec(dllexport) void WINAPI  opensourcebdm_write_reg_sp(unsigned int value) {
	usb_data[0]=3;	 /* send 3 bytes */
	usb_data[1]=CMD_WRITE_REG_SP;
	*((unsigned int*)(usb_data+2)) = value;
	opensourcebdm_usb_send_ep0(usb_data);
}

/* writes H:X (HCS08) or IX (HC12) */
__declspec(dllexport) void WINAPI  opensourcebdm_write_reg_x(unsigned int value) {
	usb_data[0]=3;	 /* send 3 bytes */
	usb_data[1]=CMD_WRITE_REG_X;
	*((unsigned int*)(usb_data+2)) = value;
	opensourcebdm_usb_send_ep0(usb_data);
}

/* writes B:A (HC12) or A (HCS08) */
__declspec(dllexport) void WINAPI  opensourcebdm_write_reg_d(unsigned int value) {
	usb_data[0]=3;	 /* send 3 bytes */
	usb_data[1]=CMD_WRITE_REG_D;
	*((unsigned int*)(usb_data+2)) = value;
	opensourcebdm_usb_send_ep0(usb_data);
}

/* writes CCR */

 __declspec(dllexport) void WINAPI opensourcebdm_write_reg_ccr(unsigned int value) {
	usb_data[0]=3;	 /* send 3 bytes */
	usb_data[1]=CMD_WRITE_REG_CCR;
	*((unsigned int*)(usb_data+2)) = value;
	opensourcebdm_usb_send_ep0(usb_data);
}


#if SEL_HCS08 || SEL_RS08
__declspec(dllexport) unsigned char WINAPI  opensourcebdm_read_status(void) {
	usb_data[0]=2;
	usb_data[1]=CMD_READ_STATUS;
	opensourcebdm_usb_recv_ep0(usb_data);
	return(usb_data[1]);
}

__declspec(dllexport) void WINAPI  opensourcebdm_write_control(unsigned char value) {
	usb_data[0]=2;     ///send 2 bytes
	usb_data[1]=CMD_WRITE_CONTROL;
	usb_data[2] = value;	//place full word
	print("Start WRITE CONTROL %5x\r\n", value);
	opensourcebdm_usb_send_ep0(usb_data);
	print("End WRITE CONTROL\r\n");
}
#endif

#if SEL_HCS12
	/* writes IY (HC12 only) */
	__declspec(dllexport) void WINAPI opensourcebdm_write_reg_y(unsigned int value) {
		usb_data[0]=3;	 /* send 3 bytes */
		usb_data[1]=CMD_WRITE_REG_Y;
		*((unsigned int*)(usb_data+2)) = value;
		opensourcebdm_usb_send_ep0(usb_data);
	}
#endif


/*********************************************************************************************/
/* RUN CONTROL		                                                                 */
/*********************************************************************************************/
#if SEL_HCS08 || SEL_RS08

__declspec(dllexport) void WINAPI  opensourcebdm_write_bkpt(unsigned int value) {
	usb_data[0]=3;     ///send 3 bytes
	usb_data[1]=CMD_WRITE_BKPT;
	*((unsigned int*)(usb_data+2)) = value;	//place full word
	print("Start WRITE BKPT %5x\r\n", value);
	opensourcebdm_usb_send_ep0(usb_data);
	//printdump(usb_data, 3);
	print("End WRITE BKPT\r\n");
}

__declspec(dllexport) unsigned int WINAPI  opensourcebdm_read_bkpt(void) {
	usb_data[0]=3;		//get 3 bytes
	usb_data[1]=CMD_READ_BKPT;
	print("Start READ BKPT\r\n");
	opensourcebdm_usb_recv_ep0(usb_data);
	print("End READ BKPT  %2x  %2x  %2x \r\n",usb_data[0],usb_data[1],usb_data[2] );

	//return(usb_data[1]);
	return((*((unsigned int *)(usb_data+1))&0xffff));
}
#endif



/*********************************************************************************************/
/* DllMain					                                                                 */
/*********************************************************************************************/

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:

			opensourcebdm_usb_init();			//initialise USB 
			#ifdef LOG
				GetModuleFileName(NULL,path,sizeof(path));
				charp=strchr(path,'\\');
				if (charp!=NULL) {
					while (strchr(charp+1,'\\')) charp=strchr(charp+1,'\\');	// find the last backslash 
					strcpy(charp+1,"OpenSourceBDM_dll.log");
					log_file=fopen(path,"wb");
				}
				print("Log file path: %s\r\n",path);
				print("Open Source BDM DLL v%1d.%1d. Compiled on %s, %s.\r\n",opensourcebdm_DLL_VERSION/16,opensourcebdm_DLL_VERSION&0x0f,__DATE__,__TIME__);
				time(&time_now);
				print("Log file created on: %s\r", ctime(&time_now));
			#endif
            break;

		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
						#ifdef LOG
				time(&time_now);
				print("End of log file: %s\r", ctime(&time_now));
				fclose(log_file);
			#endif
            break;
    }
    return TRUE;
}

/*

BOOL WINAPI  __declspec(dllexport) LibMain(HINSTANCE hDLLInst, DWORD fdwReason, LPVOID lpvReserved) {
	char path[MAX_PATH];
	char *charp;
	time_t time_now;
    switch (fdwReason) {
        case DLL_PROCESS_ATTACH:
			opensourcebdm_usb_init();			//initialise USB 
			#ifdef LOG
				GetModuleFileName(NULL,path,sizeof(path));
				charp=strchr(path,'\\');
				if (charp!=NULL) {
					while (strchr(charp+1,'\\')) charp=strchr(charp+1,'\\');	// find the last backslash 
					strcpy(charp+1,"OpenSourceBDM_dll.log");
					log_file=fopen(path,"wb");
				}
				print("Log file path: %s\r\n",path);
				print("Open Source BDM DLL v%1d.%1d. Compiled on %s, %s.\r\n",opensourcebdm_DLL_VERSION/16,opensourcebdm_DLL_VERSION&0x0f,__DATE__,__TIME__);
				time(&time_now);
				print("Log file created on: %s\r", ctime(&time_now));
			#endif
            break;
        case DLL_PROCESS_DETACH:
			#ifdef LOG
				time(&time_now);
				print("End of log file: %s\r", ctime(&time_now));
				fclose(log_file);
			#endif
            break;
        case DLL_THREAD_ATTACH:
            break;
        case DLL_THREAD_DETACH:
            break;
    }
    return TRUE;
}
*/
